package com.gic.auth.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.*;

/**
 *
 * @ClassName: ResourceApiService
 * @Description:
 * @author zhiwj
 * @date 2019/8/28 18:14
 */
public interface ResourceApiService {

    /**
     * 保存或更新资源
     * @Title: saveOrUpdateResource
     * @Description: 保存或更新资源
     * @author zhiwj
     * @param resourceDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> saveOrUpdateResource(ResourceDTO resourceDTO);

    /**
     * 删除资源
     * @Title: delResource
     * @Description: 删除资源
     * @author zhiwj
     * @param resourceId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Integer> delResource(Integer resourceId);

    /**
     * 资源组列表
     * @Title: listResource
     * @Description:
     * @author zhiwj
     * @param resourceDTO
     * @param pageNum
     * @param pageSize
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.auth.dto.ResourceDTO>>
     * @throws
     */
    ServiceResponse<Page<ResourceDTO>> listResource(ResourceDTO resourceDTO, Integer pageNum, Integer pageSize);

    /**
     * 查询资源详情
     * @Title: getResource
     * @Description:
     * @author zhiwj
     * @param resourceId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.auth.dto.ResourceDTO>>
     * @throws
     */
    ServiceResponse<ResourceDTO> getResource(Integer resourceId);

    /**
     * 按用户查询短信签名
     * @Title: getSmsResourceByUserId
     * @Description:
     * @author zhiwj
     * @param userId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.SmsSignResourceDTO>
     * @throws
     */
    ServiceResponse<SmsSignResourceDTO> getSmsResourceByUserId(Integer userId);

    /**
     * 查询用户的渠道资源(仅当前可用的数据)
     * @Title: getChannelResourceByUserId
     * @Description:
     * @author zhiwj
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<ChannelResourceDTO> getChannelResourceByUserId(Integer userId);

    /**
     * 查询用户的渠道资源(包含已经删除的数据)
     * @Title: getChannelResourceByUserIdContainDel
     * @Description:
     * @author zhiwj
     * @param userId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.ChannelResourceDTO>
     * @throws
     */
    ServiceResponse<ChannelResourceDTO> getChannelResourceByUserIdContainDel(Integer userId);

    /**
     * 查询门店资源
     * @Title: getStoreResourceByUserId
     * @Description:
     * @author zhiwj
     * @param userId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<StoreResourceDTO> getStoreResourceByUserId(Integer userId);

    /**
     * 查询订单资源
     * @Title: getOrderResourceByUserId
     * @Description:
     * @author zhiwj
     * @param userId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<OrderResourceDTO> getOrderResourceByUserId(Integer userId);

    /**
     * @Title: getUserResourceByUserId
     * @Description:
     * @author zhiwj
     * @param userId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.OrderResourceDTO> 返回null表示没有授权  返回 -1 表示全部
     * @throws
     */
    ServiceResponse<Long> getUserResourceByUserId(Integer userId);
}
