package com.gic.auth.dao.mapper;

import java.util.List;

import com.gic.auth.dto.AuthItemDTO;
import org.apache.ibatis.annotations.Param;

import com.gic.auth.dto.MenuItemDTO;
import com.gic.auth.entity.TabSysMenuItem;

public interface TabSysMenuItemMapper {
    /**
     * 根据主键删除
     *
     * @param menuItemId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer menuItemId);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabSysMenuItem record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabSysMenuItem record);

    /**
     * 根据主键查询
     *
     * @param menuItemId 主键
     * @return 实体对象
     */
    TabSysMenuItem selectByPrimaryKey(Integer menuItemId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabSysMenuItem record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabSysMenuItem record);

    /**
     * 批量新增权限项、操作项关联
     * @Title: insertForeach 
     * @Description:
      * @author guojuxing
     * @param list 
     * @return  void 
      */
    void insertForeach(@Param("list") List<MenuItemDTO> list);

    /**
     * 删除操作项根据权限项ID
     * @Title: deleteByMenuId 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  void 
      */
    void deleteByMenuId(@Param("menuId") Integer menuId);

    /**
     * 根据权限项查询操作项数据
     * @Title: listMenuItemByMenuId 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenuItem> 
      */
    List<TabSysMenuItem> listMenuItemByMenuId(@Param("menuId") Integer menuId);

    List<TabSysMenuItem> listMenuItemByMenuIdList(@Param("list") List<Integer> menuIdList);

    /**
     * 获取所有的配置在权限项内的操作项
     * @Title: getOperationItemIdList 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<java.lang.Integer> 
      */
    List<Integer> getOperationItemIdList();

    /**
     * 根据操作项ID分组查询权限项和权限项内的操作项数量
     * @Title: queryAuthItem 
     * @Description:
      * @author guojuxing
     * @param itemId 
     * @return  java.util.List<com.gic.auth.dto.AuthItemDTO> 
      */
    List<AuthItemDTO> queryAuthItem(@Param("itemId") Integer itemId);


    void deleteByMenuIdList(@Param("list") List<Integer> menuId);

    void deleteByItemId(@Param("itemId") Integer itemId);

    List<TabSysMenuItem> listByDev(Integer menuId);
}