package com.gic.auth.service;


import java.util.List;

import com.gic.auth.dto.AuthItemListDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.entity.TabSysMenu;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import com.github.pagehelper.Page;

public interface MenuService {
    List<TabSysMenu> listMenu(MenuListQO params);

    TabSysMenu getMenuById(Integer menuId);

    /**
     * 新增页面
     * @Title: savePage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  menuId 
      */
    int savePage(MenuDTO menuDTO);

    /**
     * 获取当前最大排序值
     * @Title: getMaxSortByParentIdAndLevel 
     * @Description:
      * @author guojuxing
     * @param parentId 父级ID
     * @param project
     * @return  int 
      */
    int getMaxSortByParentId(Integer parentId, String project);

    /**
     * @Title: updatePage 
     * @Description: 
      * @author guojuxing 
 * @param menuDTO 
     * @return  void 
      */
    void updatePage(MenuDTO menuDTO);

    /**
     * 查询页面下面的操作项列表数据
     * @Title: listMenuRoleByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenuItemByParentId(Integer parentId);
    /**
     * 拖拽排序
     * @Title: setSort 
     * @Description:
      * @author guojuxing
     * @param record
     * @param sortValue 
     * @return  void 
      */
    void setSort(TabSysMenu record, Integer sortValue);

    /**
     * 分页查询操作项
     * @Title: pageMenuOperationItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.github.pagehelper.Page<com.gic.auth.entity.TabSysMenu> 
      */
    Page<TabSysMenu> pageMenuOperationItem(MenuOperationItemListQO params);

    /**
     * 查询
     * @Title: selectByIds 
     * @Description:
      * @author guojuxing
     * @param menuIdList 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> selectByIds(List<Integer> menuIdList);

    /**
     * 根据ID list 查询 排序
     * @Title: selectByIdsAndOrderBy 
     * @Description:
      * @author guojuxing
     * @param menuIdList
     * @param orderByStr  order by sort 比如
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> selectByIdsAndOrderBy(List<Integer> menuIdList, String orderByStr);

    /**
     * 查询页面下的权限项数据
     * @Title: listAuthItemByMenuId 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  java.util.List<com.gic.auth.dto.AuthItemListDTO> 
      */
    List<AuthItemListDTO> listAuthItemByMenuId(Integer menuId);

    /**
     * 是否有子项目（子页面、权限项等）
     * @Title: isHasChild 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  boolean 
      */
    boolean isHasChild(Integer menuId);

    /**
     * 查询菜单树（页面&权限项）
     * @Title: listMenuTree 
     * @Description:
      * @author guojuxing
     * @param project 
     * @param versionCode gic商户基础服务版本
     * @param authTypeList 权限适用 1：管理员 2：超管可用&达摩侧 3：仅达摩侧
     * @return  java.util.List<com.gic.auth.dto.MenuDTO> 
      */
    List<TabSysMenu> listMenuTree(String project, String versionCode, List<Integer> authTypeList);

    /**
     * 根据项目查询
     * @Title: selectByProjectList 
     * @Description:
      * @author guojuxing
     * @param projectList 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> selectByProjectList(List<String> projectList);

    /**
     * 查询应用权限
     * @Title: selectAppMenuByParam 
     * @Description:
      * @author guojuxing
     * @param projectList appID List
     * @param versionList  版本/拓展包 list
     * @param authTypeList 1:普通管理员 2：超管&达摩 3：仅达摩
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> selectAppMenuByParam(List<String> projectList, List<String> versionList, List<Integer> authTypeList);

    /**
     * 是否重复页面code
     * @Title: isRepeatMenuCode 
     * @Description:
      * @author guojuxing
     * @param code 
     * @return  boolean 
      */
    boolean isRepeatMenuCode(String code, Integer menuId);


    TabSysMenu getMenuByMenuCode(String menuCode);

    /**
     * 批量更新子集的父级code数据，根据父级ID
     * @Title: updateParentCodeByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId  父级ID
     * @return  void 
      */
    void updateParentCodeByParentId(Integer parentId, String parentCode);

    /**
     * 查询仅超管可用的操作项
     * @Title: listAdminOnlyOperationItem 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.auth.entity.TabSysUser> 
      */
    List<TabSysMenu> listAdminOnlyOperationItem();

    void deleteByMenuIdList(List<Integer> menuIdList);

    List<TabSysMenu> listByParentMenuIdList(List<Integer> parentMenuIdList);

    /**
     * 逻辑删除不存在的应用菜单
     * @Title: deleteNotInApp 
     * @Description:
     * @author guojuxing
     * @param projectList 
     * @return  void 
     */
    void deleteNotInApp(List<String> projectList);



    /***********同步测试环境数据到生产************/
    /**
     * 查询临时菜单
     * @Title: listMenuTemp 
     * @Description:
     * @author guojuxing
     * @param project 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
     */
    List<TabSysMenu> listMenuTemp(String project);

    /**
     * 查询临时gic菜单，某一个一级菜单下面
     * @Title: listMenuTempForGic 
     * @Description:
     * @author guojuxing
     * @param menuCode 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
     */
    List<TabSysMenu> listMenuTempForGic(String menuCode);

    /**
     * 查询生产表的最新ID
     * @Title: getAuthItemByDevMenuId 
     * @Description:
     * @author guojuxing
     * @param menuId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
     */
    List<TabSysMenu> getAuthItemByDevMenuId(Integer menuId);
}
