package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.config.Config;
import com.gic.auth.dto.AuditorDTO;
import com.gic.auth.entity.TabAuditor;
import com.gic.auth.entity.TabAuditorAuditedGroupRel;
import com.gic.auth.entity.TabAuditorProjectItemRel;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.qo.AuditorListQO;
import com.gic.auth.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.ProjectItemDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.ProjectItemQO;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.ProjectItemApiService;
import com.gic.member.api.service.ServiceUserApiService;
import com.gic.wechat.business.api.service.fwh.WeixinBaseFunService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-09-19 14:26
 */
@Service("auditorApiService")
public class AuditorApiServiceImpl implements AuditorApiService {

    private Logger logger = LogManager.getLogger(AuditorApiServiceImpl.class);

    @Autowired
    private AuditorService auditorService;
    @Autowired
    private AuditorProjectItemRelService auditorProjectItemRelService;
    @Autowired
    private AuditorAuditedGroupRelService auditorAuditedGroupRelService;
    @Autowired
    private AuditedGroupUserRelService auditedGroupUserRelService;
    @Autowired
    private ProjectItemApiService projectItemApiService;
    @Autowired
    private AuditedGroupService auditedGroupService;
    @Autowired
    private WeixinBaseFunService weixinBaseFunService;
    @Autowired
    private ServiceUserApiService serviceUserApiService;
    @Autowired
    private UserService userService;
    @Autowired
    private Config config;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<String> saveOrUpdateAuditor(AuditorDTO auditorDTO) {
        boolean nameIsRepeat = auditorService.validAuditorNameIsRepeat(auditorDTO.getEnterpriseId(), auditorDTO.getAuditorName(), auditorDTO.getAuditorId());
        if (nameIsRepeat) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "审核员姓名重复");
        }
        boolean phoneIsRepeat = auditorService.validPhoneIsRepeat(auditorDTO.getEnterpriseId(), auditorDTO.getPhone(), auditorDTO.getAuditorId());
        if (phoneIsRepeat) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "审核员手机号重复");
        }
        boolean projectItemIsNotExist = validProjectItemIsNotExist(auditorDTO.getProjectItemList());
        if (projectItemIsNotExist) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        boolean auditedGroupIsNotExist = auditedGroupService.validAuditedGroupIsNotExist(auditorDTO.getAuditedGroupIdList());
        if (auditedGroupIsNotExist) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        if (auditorDTO.getUserId() != null) {
            TabSysUser user = userService.getUserById(auditorDTO.getUserId());
            if (user == null) {
                return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
            }
            if (!StringUtils.equals(auditorDTO.getPhone(), user.getPhoneNumber())) {
                return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "关联管理员手机号不一致");
            }
        }

        auditorDTO.setIsUse(auditorDTO.getUserId() == null ? 0 : 1);
        // 这里进来的 不会去修改openid，所以把openid设为null
        auditorDTO.setOpenid(null);
        // 保存审核员信息
        if (auditorDTO.getAuditorId() == null) {
            // save
            Integer auditorId = auditorService.save(auditorDTO);
            auditorDTO.setAuditorId(auditorId);
        } else {
            // update
            Integer line = auditorService.update(auditorDTO);
            if (line == 0) {
                return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
            }
        }
        // 受审项目
        auditorProjectItemRelService.addOrUpdateRel(auditorDTO.getEnterpriseId(), auditorDTO.getAuditorId(), auditorDTO.getProjectItemList());
        // 受审组
        auditorAuditedGroupRelService.addOrUpdateRel(auditorDTO.getEnterpriseId(), auditorDTO.getAuditorId(), auditorDTO.getAuditedGroupIdList());
        // 查询二维码
        TabAuditor auditor = auditorService.getAuditor(auditorDTO.getAuditorId());
        String tempQrCodeUrl = null;
        if (StringUtils.isBlank(auditor.getOpenid())) {
            tempQrCodeUrl = this.getTempQrCodeUrl(auditorDTO.getAuditorId());
        }
        return ServiceResponse.success(tempQrCodeUrl);
    }

    private boolean validProjectItemIsNotExist(List<Integer> projectItemList) {
        ProjectItemQO qo = new ProjectItemQO();
        qo.setProjectItemList(projectItemList);
        ServiceResponse<Page<ProjectItemDTO>> serviceResponse = projectItemApiService.listProjectItem(qo);
        return !serviceResponse.isSuccess() || serviceResponse.getResult().getTotalCount() != projectItemList.size();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> delAuditor(Integer auditorId) {
        // 删除审核员
        Integer line = auditorService.delAuditor(auditorId);
        if (line == 0) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        // 删除受审项目关联
        auditorProjectItemRelService.delRelByAuditorId(auditorId);
        // 删除受审组关联
        auditorAuditedGroupRelService.delRelByAuditorId(auditorId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<AuditorDTO> getAuditor(Integer auditorId) {
        TabAuditor auditor = this.auditorService.getAuditor(auditorId);
        if (auditor == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        AuditorDTO auditorDTO = EntityUtil.changeEntityByJSON(AuditorDTO.class, auditor);
        List<TabAuditorProjectItemRel> projectList = auditorProjectItemRelService.listRelByAuditorId(auditorId);
        if (CollectionUtils.isNotEmpty(projectList)) {
            List<Integer> projectIds = projectList.stream().map(TabAuditorProjectItemRel::getProjectItemId).collect(Collectors.toList());
            auditorDTO.setProjectItemList(projectIds);
        }
        List<TabAuditorAuditedGroupRel> groupList = auditorAuditedGroupRelService.listRelByAuditorId(auditorId);
        if (CollectionUtils.isNotEmpty(groupList)) {
            List<Integer> groupIds = groupList.stream().map(TabAuditorAuditedGroupRel::getAuditedGroupId).collect(Collectors.toList());
            auditorDTO.setAuditedGroupIdList(groupIds);
        }

        String codeUrl = this.getTempQrCodeUrl(auditorId);
        auditorDTO.setHeadUrl(codeUrl);
        // 如果没有绑定  就展示二维码， 如果绑定了  就展示微信名和头像
//        if (StringUtils.isBlank(auditor.getOpenid())) {
//            String codeUrl = this.getTempQrCodeUrl(auditorId);
//            auditorDTO.setHeadUrl(codeUrl);
//        } else {
//            ServiceResponse<ServiceUserDTO> service = this.serviceUserApiService.getByOpenid(auditorDTO.getOpenid(), auditorDTO.getEnterpriseId());
//            if (service.isSuccess()) {
//                ServiceUserDTO userDTO = service.getResult();
//                if (userDTO != null) {
//                    String headUrl = userDTO.getHeadUrl();
//                    auditorDTO.setHeadUrl(headUrl);
//                    String nickname = userDTO.getNickname();
//                    auditorDTO.setWeChatNickName(nickname);
//                } else {
//                    logger.info("openid:{}, enterpriseId:{} 查询不到该openid对应的信息", auditorDTO.getOpenid(), auditorDTO.getEnterpriseId());
//                }
//            }else{
//                logger.warn("调用serviceUserApiService.getByOpenid 服务失败,openid:{}, msg:{}", auditorDTO.getOpenid(), service.getMessage());
//            }
//        }

        return ServiceResponse.success(auditorDTO);
    }

    @Override
    public ServiceResponse<Page<AuditorDTO>> pageAuditor(AuditorListQO auditorListQO) {
        com.github.pagehelper.Page<TabAuditor> page = this.auditorService.pageAuditor(auditorListQO);
        Page<AuditorDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, AuditorDTO.class);
        if (CollectionUtils.isNotEmpty(resultPage.getResult())) {
            List<Integer> auditorIdList = resultPage.getResult().stream().map(AuditorDTO::getAuditorId).collect(Collectors.toList());
            Map<Integer, Integer> projectItemMap = auditorProjectItemRelService.getCountByAuditorIds(auditorIdList);
            Map<Integer, Integer> userCountMap = auditorAuditedGroupRelService.getUserCountByAuditorIds(auditorIdList);
            for (AuditorDTO auditorDTO : resultPage.getResult()) {
                auditorDTO.setProjectItemCount(projectItemMap.get(auditorDTO.getAuditorId()));
                auditorDTO.setAuditedGroupCount(userCountMap.get(auditorDTO.getAuditorId()));
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Void> receiveOpenid(String qrCodeParam, String openid) {
        logger.info("二维码参数：{},openid:{}", qrCodeParam, openid);
        if (StringUtils.isNotBlank(qrCodeParam) && StringUtils.isNotBlank(openid)) {
            if (qrCodeParam.startsWith("auditor_")) {
                String auditorIdStr = qrCodeParam.substring(qrCodeParam.indexOf("auditor_") + 8);
                Integer auditorId = Integer.valueOf(auditorIdStr);
                TabAuditor auditor = this.auditorService.getAuditor(auditorId);
                if (auditor != null) {
                    boolean repeat = this.auditorService.validOpenidIsRepeat(auditor.getEnterpriseId(), openid, null);
                    if (repeat) {
                        logger.warn("receiveOpenid失败，商户：{}下，存在重复的openid：{}", auditor.getEnterpriseId(), openid);
                    } else {
                        AuditorDTO auditorDTO = new AuditorDTO();
                        auditorDTO.setAuditorId(auditorId);
                        auditorDTO.setOpenid(openid);
                        auditorDTO.setIsUse(1);
                        this.auditorService.update(auditorDTO);
                    }
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<AuditorDTO>> listAuditor(AuditorListQO auditorListQO) {
        List<TabAuditor> list = this.auditorService.listAuditor(auditorListQO);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(AuditorDTO.class, list));
    }

    @Override
    public ServiceResponse<List<AuditorDTO>> listAuditorByProject(Integer projectItemId, Integer userId, Integer enterpriseId) {
        ServiceResponse<ProjectItemDTO> projectItemResponse = projectItemApiService.getById(projectItemId);
        if (projectItemResponse.isSuccess()) {
            // 找到审批项对应的审核员
            List<Integer> projectAuditorIds = auditorProjectItemRelService.listAuditorIdByProjectItemId(enterpriseId, projectItemId);
            if (CollectionUtils.isNotEmpty(projectAuditorIds)) {
                // 找到管理员所在的受审组
                List<Integer> auditedGroupIds = auditedGroupUserRelService.listAuditedGroupByUserId(enterpriseId, userId);
                if (CollectionUtils.isNotEmpty(auditedGroupIds)) {
                    List<Integer> auditorIds = auditorAuditedGroupRelService.listRelByAuditedGroupIdAndAuditId(enterpriseId, auditedGroupIds, projectAuditorIds);
                    if (CollectionUtils.isNotEmpty(auditorIds)) {
                        AuditorListQO listQO = new AuditorListQO();
                        listQO.setEnterpriseId(enterpriseId);
                        listQO.setIsUse(GlobalInfo.DATA_STATUS_NORMAL);
                        listQO.setAuditorIdList(auditorIds);
                        List<TabAuditor> auditorList = this.auditorService.listAuditor(listQO);
                        if (CollectionUtils.isNotEmpty(auditorList)) {
                            return ServiceResponse.success(EntityUtil.changeEntityListByJSON(AuditorDTO.class, auditorList));
                        }
                    }
                }
            }
            AuditorDTO adminAuditor = this.getAdminAuditor(enterpriseId);
            return ServiceResponse.success(Collections.singletonList(adminAuditor));
        } else {
            return ServiceResponse.failure(projectItemResponse.getCode(), projectItemResponse.getMessage());
        }
    }

    @Override
    public ServiceResponse<List<AuditorDTO>> listAuditorByProject(String projectItemCode, Integer userId, Integer enterpriseId) {
        ServiceResponse<ProjectItemDTO> projectItemResponse = this.projectItemApiService.getByProjectItemCode(projectItemCode);
        if (!projectItemResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        ProjectItemDTO itemDTO = projectItemResponse.getResult();
        return this.listAuditorByProject(itemDTO.getProjectItemId(), userId, enterpriseId);
    }

    @Override
    public ServiceResponse<List<AuditorDTO>> listAuditorByProject(String projectItemCode, Integer enterpriseId) {
        ServiceResponse<ProjectItemDTO> projectItemResponse = this.projectItemApiService.getByProjectItemCode(projectItemCode);
        if (!projectItemResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        Integer projectItemId = projectItemResponse.getResult().getProjectItemId();
        // 找到审批项对应的审核员
        List<Integer> projectAuditorIds = auditorProjectItemRelService.listAuditorIdByProjectItemId(enterpriseId, projectItemId);
        if (CollectionUtils.isNotEmpty(projectAuditorIds)) {
            AuditorListQO listQO = new AuditorListQO();
            listQO.setEnterpriseId(enterpriseId);
            listQO.setIsUse(GlobalInfo.DATA_STATUS_NORMAL);
            listQO.setAuditorIdList(projectAuditorIds);
            List<TabAuditor> auditorList = this.auditorService.listAuditor(listQO);
            if (CollectionUtils.isNotEmpty(auditorList)) {
                return ServiceResponse.success(EntityUtil.changeEntityListByJSON(AuditorDTO.class, auditorList));
            }
        }
        AuditorDTO adminAuditor = this.getAdminAuditor(enterpriseId);
        return ServiceResponse.success(Collections.singletonList(adminAuditor));
    }

    @Override
    public ServiceResponse<AuditorDTO> getAuditorByOpenid(String openid) {
        TabAuditor auditor = this.auditorService.getAuditorByOpenid(openid);
        if (auditor != null) {
            return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(AuditorDTO.class, auditor));
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

    @Override
    public ServiceResponse<Void> delRelByProjectItemId(Integer projectItemId) {
        auditorProjectItemRelService.delRelByProjectItemId(projectItemId);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<AuditorDTO> getAuditorByUserId(Integer enterpriseId, Integer userId) {
        TabAuditor auditor = auditorService.getAuditorByUserId(enterpriseId, userId);
        return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(AuditorDTO.class,  auditor));
    }

    private String getTempQrCodeUrl(Integer auditorId) {
        logger.info("生成临时带参二维码 auditorId:{}", auditorId);
        System.out.println(config.getAppkey());
        ServiceResponse<String> serviceResponse = weixinBaseFunService.getTemporaryQRcodeURL(config.getAppkey(), "auditor_" + auditorId, 300);
        if (serviceResponse.isSuccess()) {
            logger.info("临时带参二维码生成成功:{}", serviceResponse.getResult());
            return serviceResponse.getResult();
        } else {
            logger.warn("临时带参二维码生成失败:{}", serviceResponse.getMessage());
            return null;
        }
        //        return "qrCodeUrl";
    }

    private AuditorDTO getAdminAuditor(Integer enterpriseId) {
        TabSysUser adminUser = userService.getUserByEnterpriseId(enterpriseId);
        TabAuditor auditor = auditorService.getAuditorByUserId(enterpriseId, adminUser.getUserId());
        if (auditor == null) {
            // 如果超级管理员不是审核员， 就封装一个
            AuditorDTO auditorDTO = new AuditorDTO();
            auditorDTO.setEnterpriseId(adminUser.getEnterpriseId());
            auditorDTO.setAuditorName(adminUser.getUserName());
            auditorDTO.setPhone(adminUser.getPhoneNumber());
            auditorDTO.setUserId(adminUser.getUserId());
            return auditorDTO;
        } else {
            return EntityUtil.changeEntityByJSON(AuditorDTO.class, auditor);
        }
    }
}
