package com.gic.auth.service.outer.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.gic.auth.dto.UserDTO;
import com.gic.auth.entity.*;
import com.gic.auth.qo.UserMenuQO;
import com.gic.authcenter.api.dto.GicDepartmentDTO;
import com.gic.authcenter.api.dto.GicPositionDTO;
import com.gic.authcenter.api.dto.GicUserDTO;
import com.gic.authcenter.api.service.GicDepartmentService;
import com.gic.authcenter.api.service.GicUserService;
import com.gic.commons.util.CollectionUtil;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseResourceRelDTO;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.open.api.dto.*;
import com.gic.open.api.service.ServeApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.*;
import com.gic.auth.dto.*;
import com.gic.auth.qo.AuthItemSaveQO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import com.gic.auth.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.open.api.service.AppTokenApiService;
import com.gic.open.api.service.ApplicationApiService;

@Service("menuApiService")
public class MenuApiServiceImpl implements MenuApiService {
    private final static Logger LOGGER = LogManager.getLogger(MenuApiServiceImpl.class);
    @Autowired
    private MenuService menuService;
    @Autowired
    private ApplicationApiService applicationApiService;
    @Autowired
    private MenuItemService menuItemService;
    @Autowired
    private UserRoleService userRoleService;
    @Autowired
    private UserService userService;
    @Autowired
    private MenuRoleService menuRoleService;
    @Autowired
    private AppTokenApiService appTokenApiService;
    @Autowired
    private ServeApiService serveApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private BusinessFrontResService businessFrontResService;
    @Autowired
    private MenuAuthDepartService menuAuthDepartService;
    @Autowired
    private GicUserService gicUserService;
    @Autowired
    private GicDepartmentService gicDepartmentService;

    @Override
    public ServiceResponse<List<MenuDTO>> listByMenuIdList(List<Integer> menuIdList) {
        return ServiceResponse
                .success(EntityUtil.changeEntityListNew(MenuDTO.class, menuService.selectByIds(menuIdList)));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listByParentMenuIdList(List<Integer> parentMenuIdList) {
        return ServiceResponse.success(
                EntityUtil.changeEntityListNew(MenuDTO.class, menuService.listByParentMenuIdList(parentMenuIdList)));
    }

    @Override
    public ServiceResponse<Set<String>> getAppIdSetByUserId(Integer userId) {
        TabSysUser tabSysUser = userService.getUserById(userId);
        if (tabSysUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户不存在");
        }
        if (tabSysUser.getSuperAdmin().intValue() == 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "超级管理员拥有全部");
        }
        List<TabSysMenu> menuList = getUserMenuList(userId);
        Set<String> resultSet;
        if (CollectionUtils.isNotEmpty(menuList)) {
            resultSet = new HashSet<>(menuList.stream()
                    .filter(e -> !e.getProject().equals("gic"))
                    .filter(e -> e.getLevel().intValue() == 1)
                    .map(e -> e.getProject()).collect(Collectors.toList()));
        } else {
            resultSet = new HashSet<>();
        }
        return ServiceResponse.success(resultSet);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listUserMenu(Integer userId, String versionCode) {
        TabSysUser tabSysUser = userService.getUserById(userId);
        if (tabSysUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户不存在");
        }

        if (StringUtils.isBlank(versionCode)) {
            return ServiceResponse.success(new ArrayList<>());
        }
        //是否是运维用户
        boolean isOperationUser = tabSysUser.getLoginType().intValue() != 0;

        List<MenuDTO> result;
        if (tabSysUser.getSuperAdmin().intValue() == 1) {
            List<TabSysMenu> temp = new ArrayList<>();
            //超级管理员
            MenuListQO params = new MenuListQO();
            if (!isOperationUser) {
                params.setAuthTypeList(Arrays.stream(new Integer[]{1,2}).collect(Collectors.toList()));
            }
            //查询gic的菜单
            params.setIsGIC(MenuProjectConstants.DEFAULT_PROJECT_CODE);
            params.setVersionCode(versionCode);
            params.setMenuType(MenuTypeEnum.PAGE.getCode());
            temp.addAll(this.menuService.listMenu(params));
            //查询应用的菜单
            params.setVersionCode("");
            params.setIsGIC(MenuProjectConstants.APP_PROJECT_CODE);
            params.setMenuType(MenuTypeEnum.PAGE.getCode());
            temp.addAll(this.menuService.listMenu(params));

            result = EntityUtil.changeEntityListNew(MenuDTO.class, filterOperationUserDepartAuth(temp, tabSysUser));
        } else {
            List<TabSysMenu> menuList = getUserMenuList(userId);
            result = EntityUtil.changeEntityListNew(MenuDTO.class, menuList);
            ServiceResponse<List<MenuDTO>> tempResult = getNoAuthOperationItem(userId, MenuProjectConstants.ALL_CODE,
                    versionCode);
            result.addAll(tempResult.getResult());
        }

        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getUserMenuOfGic(Integer enterpriseId, Integer userId, Integer currentMenu, String versionCode) {
        return getUserLoginMenu(new UserMenuQO()
                .setEnterpriseId(enterpriseId)
                .setUserId(userId)
                .setGic(true)
                .setBaseVersionCode(versionCode)
                .setNeedTree(true));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getUserMenuOfGic(UserMenuQO params) {
        return getUserLoginMenu(params);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getUserMenuOfGicNotTree(Integer enterpriseId, Integer userId, Integer currentMenu,
                                                                  String versionCode) {
        return getUserLoginMenu(new UserMenuQO()
                .setEnterpriseId(enterpriseId)
                .setUserId(userId)
                .setGic(true)
                .setBaseVersionCode(versionCode)
                .setNeedTree(false));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getCollaboratorAppMenuPage(Integer enterpriseId, List<Integer> menuIdList, String appId) {
        return getAppMenuPage(enterpriseId, menuIdList, appId);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getCollaboratorAppOperationItem(List<Integer> menuIdList, String appId, Integer userId) {
        return getAppNoAuthOperationItem(menuIdList, appId, userId);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getUserMenuOfApp(Integer enterpriseId, Integer userId, Integer currentMenu,
                                                           String versionCode) {
        return getUserLoginMenu(new UserMenuQO()
                .setEnterpriseId(enterpriseId)
                .setUserId(userId)
                .setGic(false)
                .setAppId(versionCode)
                .setNeedTree(true));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getUserMenuOfAppNotTree(Integer enterpriseId, Integer userId, Integer currentMenu,
                                                                  String versionCode) {
        return getUserLoginMenu(new UserMenuQO()
                .setEnterpriseId(enterpriseId)
                .setUserId(userId)
                .setGic(false)
                .setAppId(versionCode)
                .setNeedTree(false));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getNoAuthOperationItemOfGic(Integer userId, String versionCode) {
        return getNoAuthOperationItem(userId, MenuProjectConstants.DEFAULT_PROJECT_CODE, versionCode);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getNoAuthOperationItemOfApp(Integer userId, String versionCode) {
        return getNoAuthOperationItem(userId, MenuProjectConstants.APP_PROJECT_CODE, versionCode);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getSuperAdminGicMenu(Integer enterpriseId) {
        //查询超级管理员
        TabSysUser tabSysUser = userService.getUserByEnterpriseId(enterpriseId);
        if (tabSysUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有超级管理员，数据错误");
        }
        String versionCode = enterpriseApiService.getEnterpriseById(enterpriseId).getResult().getVersionCode();
        return this.getUserMenuOfGic(enterpriseId, tabSysUser.getUserId(), null, versionCode);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getAllMenuNotTree(String versionCode) {
        List<TabSysMenu> temp = new ArrayList<>();
        //超级管理员
        MenuListQO params = new MenuListQO();
        //查询gic的菜单
        params.setIsGIC(MenuProjectConstants.DEFAULT_PROJECT_CODE);
        params.setVersionCode(versionCode);
        params.setMenuType(MenuTypeEnum.PAGE.getCode());
        temp.addAll(this.menuService.listMenu(params));
        //查询应用的菜单
        params.setVersionCode("");
        params.setIsGIC(MenuProjectConstants.APP_PROJECT_CODE);
        params.setMenuType(MenuTypeEnum.PAGE.getCode());
        temp.addAll(this.menuService.listMenu(params));
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, temp));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getSuperAdminMenuNotTree(Integer enterpriseId) {
        ServiceResponse<EnterpriseDTO> result = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (result.isSuccess()) {
            String versionCode = result.getResult().getVersionCode();
            return getAllMenuNotTree(versionCode);
        }
        return ServiceResponse.failure(result.getCode(), result.getMessage());
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuTree(MenuListQO params) {
        if (params.getIsGIC() == null) {
            params.setIsGIC(MenuProjectConstants.DEFAULT_PROJECT_CODE);
        }
        //应用菜单类型
        if (params.getIsGIC() == MenuProjectConstants.APP_PROJECT_CODE && params.isNeedUpdateAppMenu()) {
            saveAppMenu();
        }
        List<TabSysMenu> menuList = this.menuService.listMenu(params);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, menuList));
    }

    @Override
    public ServiceResponse<List<MenuAuthDepartDTO>> listAllAuthDepart(List<String> menuCodeList) {
        List<TabSysMenuAuthDepart> list = menuAuthDepartService.listAllAuthDepart(menuCodeList);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuAuthDepartDTO.class, list));
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuTreeForRole(String project, String versionCode) {
        //普通管理员级别的菜单，给子管理员适用
        List<TabSysMenu> list = menuService.listMenuTree(project, versionCode, Arrays.asList(1));
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, list));
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuTreeForRoleOfGic(String versionCode) {
        if (StringUtils.isBlank(versionCode)) {
            return ServiceResponse.success(new ArrayList<>());
        }
        return listMenuTreeForRole(MenuProjectConstants.DEFAULT_PROJECT, versionCode);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuItemByParentId(Integer parentId) {
        TabSysMenu record = menuService.getMenuById(parentId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，页面不存在");
        }
        List<TabSysMenu> menuList = menuService.listMenuItemByParentId(parentId);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, menuList));
    }

    @Override
    public ServiceResponse<Void> saveGICPage(MenuDTO menuDTO) {
        menuDTO.setProject(MenuProjectConstants.DEFAULT_PROJECT);
        return savePage(menuDTO, MenuDTO.SavePageValid.class);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listAppPageOfHasBuy(Integer enterpriseId, Integer userId) {
        ServiceResponse<List<ApplicationTokenDTO>> appResult = appTokenApiService
                .listApplicationByeId(Long.valueOf(enterpriseId));
        if (appResult.isSuccess()) {
            List<ApplicationTokenDTO> appList = appResult.getResult();
            if (CollectionUtils.isNotEmpty(appList)) {
                //调用所有的应用列表保存接口，保证，已购买的应用已经在菜单
                saveAppMenu();
                //查询已购买的有权限的app应用菜单
                List<String> projectList = new ArrayList<>(appList.size());
                List<String> versionList = new ArrayList<>();
                for (ApplicationTokenDTO app : appList) {
                    projectList.add(app.getApplicationId().toString());
                    versionList.add(app.getVersionCode());
                    if (CollectionUtils.isNotEmpty(app.getExpanList())) {
                        for (ApplicationExpanDTO expanDTO : app.getExpanList()) {
                            versionList.add(expanDTO.getAppExpanId().toString());
                        }
                    }
                }

                List<TabSysMenu> tabSysMenuList;
                if (userId != null) {
                    //过滤运营人员部门职位权限
                    TabSysUser tabSysUser = userService.getUserById(userId);
                    if (tabSysUser == null) {
                        return ServiceResponse.success(new ArrayList<>());
                    }
                    if (tabSysUser.getLoginType().intValue() != 0) {
                        tabSysMenuList = filterOperationUserDepartAuth(menuService
                                .selectAppMenuByParam(projectList, versionList, null), tabSysUser);
                    } else if (tabSysUser.getSuperAdmin().intValue() == 1) {
                        //超级管理员，包括1和2的权限适用
                        tabSysMenuList = menuService.selectAppMenuByParam(projectList,
                                versionList,
                                Arrays.stream(new Integer[]{1, 2}).collect(Collectors.toList()));
                    } else {
                        tabSysMenuList = menuService.selectAppMenuByParam(projectList, versionList, Arrays.asList(1));
                    }
                } else {
                    //过滤
                    tabSysMenuList = menuService.selectAppMenuByParam(projectList, versionList, Arrays.asList(1));
                }

                List<MenuDTO> menuList = EntityUtil.changeEntityListNew(MenuDTO.class,
                        tabSysMenuList);
                return ServiceResponse.success(menuList);
            } else {
                return ServiceResponse.success(new ArrayList<>());
            }
        }
        return ServiceResponse.failure(appResult.getCode(), appResult.getMessage());
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listAppPageOfHasBuyForRole(Integer enterpriseId) {
        return listAppPageOfHasBuy(enterpriseId, null);
    }

    @Override
    public ServiceResponse<Void> updateGICPage(MenuDTO menuDTO) {
        return updatePage(menuDTO, MenuTypeEnum.PAGE, MenuDTO.SavePageValid.class);
    }

    @Override
    public ServiceResponse<Void> deleteMenu(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (record.getMenuType().intValue() == MenuTypeEnum.PAGE.getCode()) {
            //如果是页面，判断下面是否有子页面
            if (menuService.isHasChild(menuId)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面下有项目，不允许删除词页面");
            }
        } else if (record.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode()) {
            //权限项，需要同时删除关联表
            menuItemService.deleteByMenuId(menuId);
        }

        MenuDTO menuDTO = new MenuDTO();
        menuDTO.setMenuId(menuId);
        menuDTO.setStatus(0);
        menuService.updatePage(menuDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> saveGICChildPage(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveGICChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateGICChildPage(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuTypeEnum.PAGE, MenuDTO.UpdateGICChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateAppPage(MenuDTO menuDTO) {
        //只修改页面code
        TabSysMenu record = menuService.getMenuById(menuDTO.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (record.getLevel().intValue() != MenuLevelConstants.FIRST_LEVEL) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非第一级页面");
        }
        if (record.getMenuType().intValue() != MenuTypeEnum.PAGE.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "主键参数有误，非" + MenuTypeEnum.PAGE.getMessage());
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, MenuDTO.UpdateAppPageValid.class);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //验证code是否重复
        if (menuService.isRepeatMenuCode(menuDTO.getMenuCode(), menuDTO.getMenuId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "code:" + menuDTO.getMenuCode() + "重复");
        }

        setMenuAuthType(menuDTO);
        menuService.updatePage(menuDTO);
        menuAuthDepartService.saveFetch(menuDTO.getMenuAuthDepartList(), menuDTO.getMenuCode());

        //同时更新子集的父级code数据
        menuService.updateParentCodeByParentId(menuDTO.getMenuId(), menuDTO.getMenuCode());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> saveAppChildPage(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveAppChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateAppChildPage(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuTypeEnum.PAGE, MenuDTO.EditAppChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> setSort(Integer id, Integer sort) {
        TabSysMenu record = menuService.getMenuById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (sort == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "排序值不能为空");
        }
        menuService.setSort(record, sort);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> saveOperationItem(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.OPERATION.getCode());
        ServiceResponse<Integer> saveOperationItemResult = saveChildPage(menuDTO, MenuDTO.SaveOperationItemValid.class);
        if (saveOperationItemResult.isSuccess()) {
            if (menuDTO.getAdminOnlySign() != null
                    && menuDTO.getAdminOnlySign().intValue() == MenuOperationItemConstants.ADMIN_ONLY) {
                //如果是仅超管可用，则不同步生成权限项
            } else if (menuDTO.isNeedCreateAuthItem()) {
                //同步生成权限项
                AuthItemSaveQO authItemSaveQO = new AuthItemSaveQO();
                authItemSaveQO.setMenuName(menuDTO.getMenuName());
                authItemSaveQO.setOperationItemIds(saveOperationItemResult.getResult().toString());
                authItemSaveQO.setParentId(menuDTO.getParentId());
                ServiceResponse saveAuthItemResult = saveAuthItem(authItemSaveQO);
                LOGGER.info("同步生成权限项结果:{}", JSON.toJSONString(saveAuthItemResult));
            }
        }
        return saveOperationItemResult;
    }

    @Override
    public ServiceResponse<Void> updateOperationItem(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuTypeEnum.OPERATION, MenuDTO.UpdateOperationItemValid.class);
    }

    @Override
    public ServiceResponse<Page<MenuDTO>> pageMenuOperationItem(MenuOperationItemListQO params) {
        com.github.pagehelper.Page page = menuService.pageMenuOperationItem(params);
        Page<MenuDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, MenuDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<MenuDTO> getDetailById(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        MenuDTO menuDTO = EntityUtil.changeEntityNew(MenuDTO.class, record);
        if (record.getAuthType().intValue() != 1) {
            //添加部门职位数据
            List<TabSysMenuAuthDepart> list = menuAuthDepartService
                    .listAuthDepartByMenuCode(record.getMenuCode());
            menuDTO.setMenuAuthDepartList(EntityUtil.changeEntityListNew(MenuAuthDepartDTO.class, list));
        }
        return ServiceResponse.success(menuDTO);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Void> saveAuthItem(AuthItemSaveQO params) {
        TabSysMenu record = menuService.getMenuById(params.getParentId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级页面不存在");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(params, AuthItemSaveQO.SaveAuthItemValid.class);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //新增权限项
        MenuDTO menuDTO = new MenuDTO();
        menuDTO.setMenuName(params.getMenuName());
        menuDTO.setParentId(params.getParentId());
        menuDTO.setMenuType(MenuTypeEnum.ROLE.getCode());
        menuDTO.setMenuVersion(record.getMenuVersion());
        menuDTO.setMenuVersionName(record.getMenuVersionName());
        setNormalFieldValue(menuDTO, record);
        int menuId = menuService.savePage(menuDTO);

        String operationItemIds = params.getOperationItemIds();
        String[] operationItemArr = operationItemIds.split(SignConstants.COMMA);
        //校验操作项是否存在
        List<Integer> menuIdList = new ArrayList<>(operationItemArr.length);
        for (String str : operationItemArr) {
            menuIdList.add(Integer.parseInt(str));
        }
        List<TabSysMenu> menuList = menuService.selectByIds(menuIdList);
        if (menuList == null || menuList.size() < operationItemArr.length) {
            //回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "操作项数据有误，其中有不存在的操作项");
        }
        //新增权限项、操作项关联表数据
        List<MenuItemDTO> itemDTOList = new ArrayList<>();
        Date now = new Date();
        for (String operationItem : operationItemArr) {
            MenuItemDTO menuItemDTO = new MenuItemDTO();
            menuItemDTO.setItemId(Integer.parseInt(operationItem));
            menuItemDTO.setMenuId(menuId);
            menuItemDTO.setStatus(1);
            menuItemDTO.setCreateTime(now);
            menuItemDTO.setUpdateTime(now);
            itemDTOList.add(menuItemDTO);
        }
        //delete
        menuItemService.deleteByMenuId(menuId);
        //save
        menuItemService.saveFetch(itemDTOList);
        return ServiceResponse.success();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Void> updateAuthItem(AuthItemSaveQO params) {
        TabSysMenu record = menuService.getMenuById(params.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (record.getMenuType().intValue() != MenuTypeEnum.ROLE.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非权限项");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(params, AuthItemSaveQO.EditAuthItemValid.class);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //新增权限项
        MenuDTO menuDTO = new MenuDTO();
        menuDTO.setMenuName(params.getMenuName());
        menuDTO.setMenuId(params.getMenuId());
        menuDTO.setMenuType(MenuTypeEnum.ROLE.getCode());
        setNormalFieldValue(menuDTO, record);
        menuService.updatePage(menuDTO);

        String operationItemIds = params.getOperationItemIds();
        String[] operationItemArr = operationItemIds.split(SignConstants.COMMA);
        //校验操作项是否存在
        List<Integer> menuIdList = new ArrayList<>(operationItemArr.length);
        for (String str : operationItemArr) {
            menuIdList.add(Integer.parseInt(str));
        }
        List<TabSysMenu> menuList = menuService.selectByIds(menuIdList);
        if (menuList == null || menuList.size() < operationItemArr.length) {
            //回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "操作项数据有误，其中有不存在的操作项");
        }
        //新增权限项、操作项关联表数据
        List<MenuItemDTO> itemDTOList = new ArrayList<>();
        Date now = new Date();
        for (String operationItem : operationItemArr) {
            MenuItemDTO menuItemDTO = new MenuItemDTO();
            menuItemDTO.setItemId(Integer.parseInt(operationItem));
            menuItemDTO.setMenuId(params.getMenuId());
            menuItemDTO.setStatus(1);
            menuItemDTO.setCreateTime(now);
            menuItemDTO.setUpdateTime(now);
            itemDTOList.add(menuItemDTO);
        }
        //delete
        menuItemService.deleteByMenuId(params.getMenuId());
        //save
        menuItemService.saveFetch(itemDTOList);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<AuthItemListDTO>> listAuthItemByMenuId(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，没有页面");
        }
        List<AuthItemListDTO> authItemList = menuService.listAuthItemByMenuId(menuId);
        return ServiceResponse.success(authItemList);
    }

    /**
     * 查询gic基础服务版本列表数据
     * @Title: getGicDefaultVersion 
     * @Description:
      * @author guojuxing  
     * @return  java.util.Map<java.lang.String,java.lang.String> 
      */
    @Override
    public ServiceResponse<Map<String, String>> getGicDefaultVersion() {
        Map<String, String> map = new HashMap<>(16);
        ServiceResponse<List<ServePropDTO>> versionListResult = serveApiService.listServePropByGICDefaultAPP();
        if (versionListResult.isSuccess()) {
            List<ServePropDTO> versionList = versionListResult.getResult();
            if (CollectionUtils.isNotEmpty(versionList)) {
                for (ServePropDTO servePropDTO : versionList) {
                    if ("version".equals(servePropDTO.getPropType())) {
                        map.put(servePropDTO.getCode(), servePropDTO.getName());
                    }
                }
            }
        }
        return ServiceResponse.success(map);
    }

    @Override
    public ServiceResponse<MenuDTO> getMenuByMenuCode(String menuCode) {
        TabSysMenu record = menuService.getMenuByMenuCode(menuCode);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单code错误，无数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(MenuDTO.class, record));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> test(String menuCode) {
        TabSysMenu tabSysMenu = menuService.getMenuByMenuCode(menuCode);
        List<TabSysMenu> devMenuList = menuService.listMenuTempForGic(menuCode);
        devMenuList.add(tabSysMenu);
        return ServiceResponse.success(treeMenu(devMenuList, 1, 0));
    }

    @Override
    public ServiceResponse<Void> syncProductMenu(String menuCode) {
        TabSysMenu tabSysMenu = menuService.getMenuByMenuCode(menuCode);
        if (tabSysMenu == null || tabSysMenu.getLevel() != 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在或者不是应用对应的code");
        }
        //临时表（测试环境的菜单数据）
        List<TabSysMenu> devMenuList = menuService.listMenuTempForGic(tabSysMenu.getMenuCode());
        if (CollectionUtils.isNotEmpty(devMenuList)) {
            devMenuList.forEach(menu -> {
                TabSysMenu temp = menuService.getMenuByMenuCode(menu.getMenuCode());
                if (temp == null) {
                    //如果生产环境没有，则新增；如果有了，则不处理
                    int level = menu.getLevel();
                    if (level == 1) {
                        //如果是第一层级的，应用
                        //不处理
                    } else if (level == 2) {
                        //如果是第二层级，则父级code，进行修改
                        menu.setParentId(tabSysMenu.getMenuId());
                        menu.setParentCode(tabSysMenu.getMenuCode());
                        menu.setMenuVersion(tabSysMenu.getMenuVersion());
                        menu.setMenuVersionName(tabSysMenu.getMenuVersionName());
                        menu.setProject(tabSysMenu.getProject());
                        //避免冲突
                        menu.setMenuId(null);
                        menuService.savePage(EntityUtil.changeEntityNew(MenuDTO.class, menu));
                    } else {
                        //其他层级,查询他的父级ID，根据父级code
                        TabSysMenu otherLevel = menuService.getMenuByMenuCode(menu.getParentCode());
                        menu.setParentId(otherLevel.getMenuId());
                        menu.setMenuVersion(tabSysMenu.getMenuVersion());
                        menu.setMenuVersionName(tabSysMenu.getMenuVersionName());
                        menu.setProject(tabSysMenu.getProject());
                        //避免冲突
                        menu.setMenuId(null);
                        menuService.savePage(EntityUtil.changeEntityNew(MenuDTO.class, menu));
                    }
                }
            });
            //处理完所有的菜单
            devMenuList.forEach(item -> {
                //权限项和操作项关联数据同步
                if (item.getMenuType().intValue() == 1) {
                    List<TabSysMenu> itemList = menuService.getAuthItemByDevMenuId(item.getMenuId());
                    if (CollectionUtils.isNotEmpty(itemList)) {
                        TabSysMenu proMenu = menuService.getMenuByMenuCode(item.getMenuCode());
                        List<MenuItemDTO> itemDTOList = new ArrayList<>();
                        Date now = new Date();
                        itemList.forEach(tempItem -> {
                            MenuItemDTO menuItemDTO = new MenuItemDTO();
                            menuItemDTO.setItemId(tempItem.getMenuId());
                            menuItemDTO.setMenuId(proMenu.getMenuId());
                            menuItemDTO.setStatus(1);
                            menuItemDTO.setCreateTime(now);
                            menuItemDTO.setUpdateTime(now);
                            itemDTOList.add(menuItemDTO);
                        });
                        //delete
                        menuItemService.deleteByMenuId(proMenu.getMenuId());
                        //save
                        menuItemService.saveFetch(itemDTOList);
                    }
                }
            });
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<MenuDTO>> filterOperationUserDepartAuth(List<MenuDTO> resultList, UserDTO tabSysUser) {
        List<TabSysMenu> list = filterOperationUserDepartAuth(EntityUtil.changeEntityListNew(TabSysMenu.class, resultList),
                EntityUtil.changeEntityNew(TabSysUser.class, tabSysUser));
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, list));
    }

    private ServiceResponse<Void> savePage(MenuDTO menuDTO, Class clazz) {
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //验证code是否重复
        if (menuService.isRepeatMenuCode(menuDTO.getMenuCode(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "code:" + menuDTO.getMenuCode() + "重复");
        }

        setMenuVersion(menuDTO);
        setNormalFieldValue(menuDTO);
        setMenuAuthType(menuDTO);
        menuService.savePage(menuDTO);
        menuAuthDepartService.saveFetch(menuDTO.getMenuAuthDepartList(), menuDTO.getMenuCode());
        return ServiceResponse.success();
    }

    /**
     * 奢姿权限适用字段数据
     * @param menuDTO
     * @return
     */
    private void setMenuAuthType(MenuDTO menuDTO) {
        Integer authType = menuDTO.getAuthType();
        if (authType == null) {
            authType = 1;
        }
        menuDTO.setAuthType(authType);
        if (authType != 1) {
            List<MenuAuthDepartDTO> list = menuDTO.getMenuAuthDepartList();
            if (CollectionUtils.isNotEmpty(list)) {
                Date now = new Date();
                menuDTO.setMenuAuthDepartList(
                        list.stream().map(e -> e.setMenuCode(menuDTO.getMenuCode())
                            .setStatus(1)
                            .setCreateTime(now)
                            .setUpdateTime(now)).collect(Collectors.toList()));
            }
        }
    }

    private ServiceResponse<Integer> saveChildPage(MenuDTO menuDTO, Class clazz) {
        TabSysMenu record = menuService.getMenuById(menuDTO.getParentId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级页面不存在");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //验证code是否重复
        if (menuService.isRepeatMenuCode(menuDTO.getMenuCode(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "code:" + menuDTO.getMenuCode() + "重复");
        }

        setNormalFieldValue(menuDTO, record);
        if (!MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            //如果是应用子页面、操作项
            if (StringUtils.isBlank(menuDTO.getMenuVersionName())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用版本名称不能改为空");
            }
        }
        setMenuVersion(menuDTO);
        setMenuAuthType(menuDTO);

        Integer menuId = menuService.savePage(menuDTO);
        menuAuthDepartService.saveFetch(menuDTO.getMenuAuthDepartList(), menuDTO.getMenuCode());
        return ServiceResponse.success(menuId);
    }

    private ServiceResponse<Void> updatePage(MenuDTO menuDTO, MenuTypeEnum menuTypeEnum, Class clazz) {
        return updateNormalPage(menuDTO, menuTypeEnum, clazz, true);
    }

    private ServiceResponse<Void> updateChildPage(MenuDTO menuDTO, MenuTypeEnum menuTypeEnum, Class clazz) {
        return updateNormalPage(menuDTO, menuTypeEnum, clazz, false);
    }

    private ServiceResponse<Void> updateNormalPage(MenuDTO menuDTO, MenuTypeEnum menuTypeEnum, Class clazz,
            boolean isFirstLevelPage) {
        TabSysMenu record = menuService.getMenuById(menuDTO.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (isFirstLevelPage) {
            if (record.getLevel().intValue() != MenuLevelConstants.FIRST_LEVEL) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非第一级页面");
            }
        } else {
            if (record.getLevel().intValue() == MenuLevelConstants.FIRST_LEVEL) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非子页面");
            }
        }
        if (record.getMenuType().intValue() != menuTypeEnum.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非" + menuTypeEnum.getMessage());
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }

        //验证code是否重复
        if (menuService.isRepeatMenuCode(menuDTO.getMenuCode(), menuDTO.getMenuId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "code:" + menuDTO.getMenuCode() + "重复");
        }

        menuDTO.setProject(record.getProject());
        if (!MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            //如果是应用子页面、操作项
            if (StringUtils.isBlank(menuDTO.getMenuVersionName())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "应用版本名称不能改为空");
            }
        }
        setMenuVersion(menuDTO);
        setMenuAuthType(menuDTO);
        menuService.updatePage(menuDTO);
        menuAuthDepartService.saveFetch(menuDTO.getMenuAuthDepartList(), menuDTO.getMenuCode());

        //同时更新子集的父级code数据
        menuService.updateParentCodeByParentId(menuDTO.getMenuId(), menuDTO.getMenuCode());
        //是否是操作项，并且编辑为仅超管可用
        //如果有权限项，
        boolean isEditToAdminOnly = menuTypeEnum.getCode() == MenuTypeEnum.OPERATION.getCode()
                && menuDTO.getAdminOnlySign() != null
                && menuDTO.getAdminOnlySign().intValue() == MenuOperationItemConstants.ADMIN_ONLY;
        if (isEditToAdminOnly) {
            //判断是否挂在某个权限项下面
            List<AuthItemDTO> authItemDTOList = menuItemService.listAuthItemGroupByItemId(record.getMenuId());
            List<Integer> authItemIdList = new ArrayList<>();
            for (AuthItemDTO authItemDTO : authItemDTOList) {
                if (authItemDTO.getOperationItemCount().intValue() <= 1) {
                    //判断下面的操作项数量，如果只有一个，则删除该权限项（因为仅超管可用，不再需要权限项）
                    authItemIdList.add(authItemDTO.getMenuId());
                }
            }
            //删除权限项关联
            menuItemService.deleteByMenuIdList(authItemIdList);
            //删除权限项
            menuService.deleteByMenuIdList(authItemIdList);
            //删除操作项
            menuItemService.deleteByItemId(record.getMenuId());
        }

        return ServiceResponse.success();
    }

    /**
     * 应用第一级页面新增
     */
    private void saveAppMenu() {
        ServiceResponse<List<ApplicationDTO>> appListResponse = applicationApiService.listApplicationAndServeProp();
        Map<String, TabSysMenu> appMenu = getFirstLevelAppMenu();
        if (appListResponse.isSuccess()) {
            List<ApplicationDTO> appList = appListResponse.getResult();
            //删除已经不存在的应用
            //获取存在的应用
            List<String> projectList = new ArrayList<>();
            for (ApplicationDTO applicationDTO : appList) {
                projectList.add(applicationDTO.getApplicationId().toString());
            }
            //删除不在projectList中的应用菜单:逻辑删除
            menuService.deleteNotInApp(projectList);
            for (ApplicationDTO app : appList) {
                if (appMenu.containsKey(app.getApplicationId().toString())) {
                    //如果已存在，则更新版本信息
                    MenuDTO menuDTO = new MenuDTO();
                    menuDTO.setMenuName(app.getName());
                    menuDTO.setProject(app.getApplicationId().toString());
                    setMenuVersion(app.getPropDTOList(), app.getServeExpanList(), menuDTO);
//                    setMenuAuthType(menuDTO);
                    menuDTO.setMenuId(appMenu.get(app.getApplicationId().toString()).getMenuId());
                    menuService.updatePage(menuDTO);
                } else {
                    MenuDTO menuDTO = new MenuDTO();
                    menuDTO.setMenuName(app.getName());
                    menuDTO.setProject(app.getApplicationId().toString());
                    setMenuVersion(app.getPropDTOList(), app.getServeExpanList(), menuDTO);
//                    setMenuAuthType(menuDTO);
                    setNormalFieldValue(menuDTO);
                    menuService.savePage(menuDTO);
                }
            }
        }
    }

    /**
     * 第一级页面通用字段值设置
     * @param menuDTO
     */
    private void setNormalFieldValue(MenuDTO menuDTO) {
        //sort
        int maxSort;
        if (MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            maxSort = menuService.getMaxSortByParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, menuDTO.getProject());
        } else {
            maxSort = menuService.getMaxSortByParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, null);
        }
        menuDTO.setSort(maxSort + 1);
        //level
        menuDTO.setLevel(1);

        menuDTO.setParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID);
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
    }

    /**
     * 新增子页面、操作项、权限项的时候设置常规字段值
     * @param menuDTO
     * @param parent 父级页面
     */
    private void setNormalFieldValue(MenuDTO menuDTO, TabSysMenu parent) {
        //project
        menuDTO.setProject(parent.getProject());
        //sort
        if (menuDTO.getMenuId() == null) {
            //新增的时候需要设置sort和level。编辑的时候不需要，因为编辑的时候不能再次选择新的父级页面，所以层级是不变的。
            int maxSort = menuService.getMaxSortByParentId(menuDTO.getParentId(), menuDTO.getProject());
            menuDTO.setSort(maxSort + 1);
            //level
            menuDTO.setLevel(parent.getLevel() + 1);
        }
        //父级code
        menuDTO.setParentCode(parent.getMenuCode());
    }

    /**
     * 获取应用第一级菜单（应用列表）
     * @return
     */
    private Map<String, TabSysMenu> getFirstLevelAppMenu() {
        MenuListQO params = new MenuListQO();
        params.setIsGIC(MenuProjectConstants.APP_PROJECT_CODE);
        params.setLevel(MenuLevelConstants.FIRST_LEVEL);
        List<TabSysMenu> menuList = menuService.listMenu(params);
        Map<String, TabSysMenu> map = new HashMap<>(16);
        if (menuList != null) {
            for (TabSysMenu menu : menuList) {
                map.put(menu.getProject(), menu);
            }
        }
        return map;
    }

    /**
     * 设置版本ID和版本名称字段值
     * @param serviceList
     * @param menuDTO
     */
    private static void setMenuVersion(List<ServePropDTO> serviceList, List<ServeExpanDTO> serveExpandList,
            MenuDTO menuDTO) {
        if (CollectionUtils.isEmpty(serviceList)) {
            return;
        }
        StringBuilder menuVersion = new StringBuilder(SignConstants.UNDERLINE);
        StringBuilder menuVersionName = new StringBuilder();
        for (ServePropDTO propDTO : serviceList) {
            menuVersion.append(propDTO.getCode()).append(SignConstants.UNDERLINE);
            menuVersionName.append(propDTO.getName()).append(SignConstants.SLASH);
        }
        if (CollectionUtils.isNotEmpty(serveExpandList)) {
            for (ServeExpanDTO serveExpanDTO : serveExpandList) {
                menuVersion.append(serveExpanDTO.getServeExpanId().toString()).append(SignConstants.UNDERLINE);
                menuVersionName.append(serveExpanDTO.getExpanName()).append(SignConstants.SLASH);
            }
        }
        menuDTO.setMenuVersion(menuVersion.toString());
        String tempMenuVersionName = menuVersionName.toString();
        menuDTO.setMenuVersionName(tempMenuVersionName.substring(0, tempMenuVersionName.length() - 1));
    }

    /**
     * 设置版本ID和版本名称字段值
     * @param menuDTO 必须有project值
     */
    private void setMenuVersion(MenuDTO menuDTO) {
        if (MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            menuDTO.setMenuVersion(SignConstants.UNDERLINE
                    + menuDTO.getMenuVersion().replaceAll(SignConstants.COMMA, SignConstants.UNDERLINE)
                    + SignConstants.UNDERLINE);
            String[] menuVersionArr = menuDTO.getMenuVersion().split(SignConstants.UNDERLINE);
            StringBuilder result = new StringBuilder();

            Map<String, String> gicDefaultVersion = getGicDefaultVersion().getResult();
            for (String str : menuVersionArr) {
                if (StringUtils.isBlank(str)) {
                    continue;
                }
                result.append(gicDefaultVersion.get(str)).append(SignConstants.SLASH);
            }
            String menuVersionStr = result.toString();
            menuDTO.setMenuVersionName(menuVersionStr.substring(0, menuVersionStr.length() - 1));
        } else {
            menuDTO.setMenuVersion(SignConstants.UNDERLINE
                    + menuDTO.getMenuVersion().replaceAll(SignConstants.COMMA, SignConstants.UNDERLINE)
                    + SignConstants.UNDERLINE);
            menuDTO.setMenuVersionName(
                    menuDTO.getMenuVersionName().replaceAll(SignConstants.COMMA, SignConstants.SLASH));
        }
    }

    private List<MenuDTO> treeMenu(List<TabSysMenu> menuList, int level, int parentId) {
        List<MenuDTO> list = null;
        if (level < 5) {
            for (TabSysMenu menu : menuList) {
                if (level == menu.getLevel() && menu.getParentId().intValue() == parentId) {
                    MenuDTO menuDTO = EntityUtil.changeEntityByJSON(MenuDTO.class, menu);
                    menuDTO.setChildren(this.treeMenu(menuList, menu.getLevel().intValue() + 1, menuDTO.getMenuId()));
                    if (list == null) {
                        list = new ArrayList<>();
                    }
                    list.add(menuDTO);
                }
            }
        }
        return list;
    }

    /**
     * 获取所有的默认操作项（即所有用户都有权限的操作项）, 默认获取gic
     * @Title: getDefaultOperationItem 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.auth.dto.MenuDTO> 
      */
    private List<MenuDTO> getDefaultOperationItem(Integer isGIC) {
        //返回结果集
        List<MenuDTO> resultList = new ArrayList<>();
        //查询权限项
        MenuListQO params = new MenuListQO();
        params.setMenuType(MenuTypeEnum.ROLE.getCode());
        if (isGIC == null) {
            params.setIsGIC(MenuProjectConstants.DEFAULT_PROJECT_CODE);
        }
        List<TabSysMenu> authItemList = this.menuService.listMenu(params);
        //查询所有已配置到权限项的操作项
        List<Integer> authItemIdList = new ArrayList<>();
        for (TabSysMenu menu : authItemList) {
            authItemIdList.add(menu.getMenuId());
        }
        List<TabSysMenuItem> menuItemList = menuItemService.listMenuItemByMenuId(authItemIdList);

        Map<Integer, String> menuItemMap = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(menuItemList)) {
            for (TabSysMenuItem menuItem : menuItemList) {
                menuItemMap.put(menuItem.getItemId(), "1");
            }
        }

        //查询所有的操作项
        params.setMenuType(MenuTypeEnum.OPERATION.getCode());
        List<TabSysMenu> operationItemList = this.menuService.listMenu(params);
        //与已配置的操作项比对，如果没有配置在权限项，说明是默认的操作项（全部用户拥有该操作项）
        for (TabSysMenu sysMenu : operationItemList) {
            if (!menuItemMap.containsKey(sysMenu.getMenuId())) {
                //如果不存在，说明是默认的
                resultList.add(EntityUtil.changeEntityNew(MenuDTO.class, sysMenu));
            }
        }
        return resultList;
    }

    /**
     * 获取配置在权限项内的操作项数据
     * @Title: getOperationItemOfAuthItem 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.auth.dto.MenuDTO> 
      */
    private List<MenuDTO> getOperationItemOfAuthItem() {
        List<Integer> operationItemIdList = menuItemService.getOperationItemIdList();
        List<TabSysMenu> operationItemList = menuService.selectByIds(operationItemIdList);
        return EntityUtil.changeEntityListNew(MenuDTO.class, operationItemList);
    }

    /**
     * 查询用户拥有的权限菜单(页面、权限项)(已经过滤登录用)
     * @Title: getUserMenuList 
     * @Description: 
      * @author guojuxing 
     * @param userId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    private List<TabSysMenu> getUserMenuList(Integer userId) {
        TabSysUser tabSysUser = userService.getUserById(userId);

        //查询用户的权限集
        List<UserRoleDTO> userRoleList = userRoleService.listUserRoleByUserId(userId);
        List<Integer> roleIdList = new ArrayList<>(userRoleList.size());
        for (UserRoleDTO userRole : userRoleList) {
            roleIdList.add(userRole.getRoleId());
        }
        //查询用户拥有的菜单
        List<MenuRoleDTO> menuRoleList = menuRoleService.listMenuRoleByRoleId(roleIdList);
        List<Integer> menuIdList = new ArrayList<>(menuRoleList.size());
        for (MenuRoleDTO menuRole : menuRoleList) {
            menuIdList.add(menuRole.getMenuId());
        }
        List<TabSysMenu> menuList = menuService.selectByIds(menuIdList);
        return filterOperationUserDepartAuth(menuList, tabSysUser);
    }

    /**
     * 获取登陆用户页面菜单
     * @Title: getUserLoginMenu 
     * @Description:
      * @author guojuxing
     * @param params
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    private ServiceResponse<List<MenuDTO>> getUserLoginMenu(UserMenuQO params) {
        boolean isNeedTree = params.isNeedTree();
        //返回结果集定义声明
        List<TabSysMenu> resultList;
        boolean isGic = params.isGic();


        //先查询登陆用户信息
        TabSysUser tabSysUser = userService.getUserById(params.getUserId());
        if (tabSysUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户不存在");
        }

        if (isGic) {
            if (StringUtils.isBlank(params.getBaseVersionCode())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户还没有订购基础服务");
            }
        }

        //是否是运维用户
        params.setOperationUser(tabSysUser.getLoginType().intValue() != 0);

        //超级用户
        MenuListQO menuListParams = new MenuListQO();
        menuListParams.setMenuType(MenuTypeEnum.PAGE.getCode());
        if (isGic) {
            menuListParams.setVersionCode(params.getBaseVersionCode());
            menuListParams.setIsGIC(MenuProjectConstants.DEFAULT_PROJECT_CODE);
        } else {
            menuListParams.setProject(params.getAppId());
            menuListParams.setIsGIC(MenuProjectConstants.APP_PROJECT_CODE);
        }

        if (params.isOperationUser()) {
            resultList = filterOperationUserDepartAuth(this.menuService.listMenu(menuListParams), tabSysUser);
        } else {

            if (tabSysUser.getSuperAdmin().intValue() == 1) {
                //超级用户
                //1:管理员权限集 2:商户级超管及达摩侧人员
                menuListParams.setAuthTypeList(Arrays.asList(new Integer[]{1, 2}));

                resultList = this.menuService.listMenu(menuListParams);
            } else {
                //用户拥有的菜单权限
                List<TabSysMenu> menuList = getUserMenuList(params.getUserId());
                List<Integer> menuIdList = new ArrayList<>();
                for (TabSysMenu menu : menuList) {
                    //符合条件的菜单ID
                    boolean isMapMenu;
                    if (isGic) {
                        isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.PAGE.getCode()
                                && menu.getProject().equals(MenuProjectConstants.DEFAULT_PROJECT);
                    } else {
                        isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.PAGE.getCode()
                                && menu.getProject().equals(params.getAppId());
                    }
                    if (isMapMenu) {
                        menuIdList.add(menu.getMenuId());
                    }
                }
                resultList = menuService.selectByIdsAndOrderBy(menuIdList, " order by sort");
            }
        }


        Map<String, Object> resourcePage = getBusinessFrontRes(params.getEnterpriseId());
        List<TabSysMenu> tempList = new ArrayList<>();
        for (TabSysMenu menu : resultList) {
            if (resourcePage.containsKey(menu.getMenuCode())) {
                //如果包含了，则过滤掉
                continue;
            }
            tempList.add(menu);
        }

        if (isNeedTree) {
            return ServiceResponse.success(this.treeMenu(tempList, 1, 0));
        } else {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, tempList));
        }
    }

    /**
     * 过滤达摩侧用户的部门职位权限
     * @param resultList
     * @param tabSysUser
     * @return
     */
    private List<TabSysMenu> filterOperationUserDepartAuth(List<TabSysMenu> resultList, TabSysUser tabSysUser) {
        if (tabSysUser != null && tabSysUser.getLoginType().intValue() != 0) {
            //运营人员的部门职位；根据手机号查询运维人员的部门职位信息
            GicUserDTO gicUserDTO = gicUserService.getUserByMobile(tabSysUser.getPhoneNumber());
            LOGGER.info("运维人员的部门职位数据:{}", JSON.toJSONString(gicUserDTO));
            //如果是运营人员
            Set<String> departPosition = new HashSet<>(2);
            String departmentCode = gicUserDTO.getDepartmentCode();
            if (StringUtils.isNotBlank(departmentCode)) {
                if (StringUtils.isBlank(gicUserDTO.getPositionCode())) {
                    //如果职位code是空，则取下面全部的职位code
                    List<GicDepartmentDTO> gicDepartmentDTOList = gicDepartmentService.listAllGicDepartment();
                    if (CollectionUtils.isNotEmpty(gicDepartmentDTOList)) {
                        Map<String, List<GicPositionDTO>> departMap = gicDepartmentDTOList
                                .stream()
                                .collect(Collectors.toMap(e -> e.getDepartmentCode(), e -> e.getPositionList()));
                        List<GicPositionDTO> positionList = departMap.get(departmentCode);
                        if (CollectionUtils.isNotEmpty(positionList)) {
                            positionList.forEach(e -> {
                                departPosition.add(departmentCode + "-" + e.getPositionCode());
                            });
                        }
                    }
                } else {
                    departPosition.add(departmentCode + "-" + gicUserDTO.getPositionCode());
                }
            }
            //过滤运营人员的部门职位权限
            Map<String, List<TabSysMenuAuthDepart>> departMap = getAuthDepartMap(null);
            return resultList.stream().filter(e -> {
                if (e.getAuthType().intValue() == 1) {
                    //普通管理员的都有效，不需要判断部门职位
                    return true;
                }
                List<TabSysMenuAuthDepart> authList = departMap.get(e.getMenuCode());
                if (authList == null) {
                    //如果是空的，说明是全部权限
                    return true;
                }
                //验证departPosition是否存在一个于authList中
                if (authList.stream().anyMatch(auth -> departPosition.contains(auth.getDepartCode() + "-" + auth.getPositionCode()))) {
                    return true;
                }
                return false;
            }).collect(Collectors.toList());
        } else if (tabSysUser != null && tabSysUser.getSuperAdmin().intValue() == 1) {
            //超级管理员
            return resultList.stream().filter(e -> e.getAuthType().intValue() != 3).collect(Collectors.toList());
        } else if (tabSysUser != null && tabSysUser.getSuperAdmin().intValue() == 0) {
            //普通管理员
            return resultList.stream().filter(e -> e.getAuthType().intValue() == 1).collect(Collectors.toList());
        }
        return resultList;
    }

    /**
     * 查询出所有的部门职位，用于过滤运营人员权限
     * @param menuCodeList
     * @return
     */
    private Map<String, List<TabSysMenuAuthDepart>> getAuthDepartMap(List<String> menuCodeList) {
        //设置部门数据，列表需要展示
        List<TabSysMenuAuthDepart> authDepartList = menuAuthDepartService
                .listAllAuthDepart(menuCodeList);
        if (CollectionUtils.isNotEmpty(authDepartList)) {
            Map<String, List<TabSysMenuAuthDepart>> authDepartMap = CollectionUtil.group(authDepartList, "menuCode");
            return authDepartMap;
        }
        return new HashMap<>(2);
    }

    private ServiceResponse<List<MenuDTO>> getAppMenuPage(Integer enterpriseId, List<Integer> menuIdList, String appId) {
        //返回结果集定义声明
        List<TabSysMenu> resultList = menuService.selectByIdsAndOrderBy(menuIdList, " order by sort");
        Map<String, Object> resourcePage = getBusinessFrontRes(enterpriseId);
        List<TabSysMenu> tempList = new ArrayList<>();
        for (TabSysMenu menu : resultList) {
            if (resourcePage.containsKey(menu.getMenuCode())) {
                //如果包含了，则过滤掉
                continue;
            }
            tempList.add(menu);
        }

        return ServiceResponse.success(this.treeMenu(tempList, 1, 0));
    }

    /**
     * 获取商户业务前置资源配置数据
     * @param enterpriseId
     * @return
     */
    private Map<String, Object> getBusinessFrontRes(Integer enterpriseId) {
        Map<String, Object> resourcePage = new HashMap<>(16);
        //过滤商品资源页面
        ServiceResponse<List<EnterpriseResourceRelDTO>> resourceResult = enterpriseApiService.getBusinessFrontRes(enterpriseId);
        if (resourceResult.isSuccess()) {
            //查询需要过滤的资源
            List<EnterpriseResourceRelDTO> resourceList = resourceResult.getResult();
            if (CollectionUtils.isNotEmpty(resourceList)) {
                List<String> resourceCodeList = new ArrayList<>();
                for (EnterpriseResourceRelDTO res : resourceList) {
                    if (res.getResourceSwitch().intValue() == 0) {
                        resourceCodeList.add(res.getResourceCode());
                    }
                }
                //如果没有开启商品资源开关，需要过滤一些菜单
                //查询配置页面
                if (CollectionUtils.isNotEmpty(resourceCodeList)) {
                    List<TabSysBusinessFrontRes> frontResourceList = businessFrontResService.listByCode(resourceCodeList, 1);
                    if (CollectionUtils.isNotEmpty(frontResourceList)) {
                        for (TabSysBusinessFrontRes resource : frontResourceList) {
                            resourcePage.put(resource.getPageName(), 1);
                        }
                    }
                }
            }
        }
        return resourcePage;
    }

    /**
     * 获取用户没有权限的操作项
     * @Title: getNoAuthOperationItem 
     * @Description:
      * @author guojuxing
     * @param userId
     * @param projectCode 
     * @param versionCode 商户基础服务版本
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    private ServiceResponse<List<MenuDTO>> getNoAuthOperationItem(Integer userId, int projectCode, String versionCode) {
        TabSysUser tabSysUser = userService.getUserById(userId);
        if (tabSysUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户不存在");
        }

        //添加逻辑：如果是超级管理员，直接全部有效，不存在没有权限的操作项
        if (tabSysUser.getSuperAdmin().intValue() == 1) {
            //
            MenuListQO params = new MenuListQO();
            List<TabSysMenu> adminMenuList = filterOperationUserDepartAuth(menuService.listMenu(params), tabSysUser);
            return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, adminMenuList));
        }

        //用户拥有的菜单权限
        List<TabSysMenu> menuList = getUserMenuList(userId);

        List<MenuDTO> resultList = getNoAuthOperationItemByMenuIdList(menuList, projectCode, versionCode);
        //仅超管可用
//        if (tabSysUser.getSuperAdmin().intValue() != 1) {
//            List<TabSysMenu> adminOnlyOperationItemList = menuService.listAdminOnlyOperationItem();
//            if (CollectionUtils.isNotEmpty(adminOnlyOperationItemList)) {
//                resultList.addAll(EntityUtil.changeEntityListNew(MenuDTO.class, adminOnlyOperationItemList));
//            }
//        }

        return ServiceResponse.success(resultList);
    }

    private List<MenuDTO> getNoAuthOperationItemByMenuIdList(List<TabSysMenu> menuList, int projectCode, String versionCode) {
        //获取用户拥有的权限项ID
        List<Integer> authItemIdList = new ArrayList<>();
        for (TabSysMenu menu : menuList) {
            boolean isMapMenu;
            if (projectCode == MenuProjectConstants.DEFAULT_PROJECT_CODE) {
                isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode()
                        && menu.getProject().equals(MenuProjectConstants.DEFAULT_PROJECT);
            } else if (projectCode == MenuProjectConstants.APP_PROJECT_CODE) {
                isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode()
                        && !menu.getProject().equals(MenuProjectConstants.DEFAULT_PROJECT);
            } else {
                isMapMenu = menu.getMenuType().intValue() == MenuTypeEnum.ROLE.getCode();
            }
            if (isMapMenu) {
                authItemIdList.add(menu.getMenuId());
            }
        }
        //如果是gic菜单，需要去判断版本code
        boolean isNeedValidVersionCode = MenuProjectConstants.DEFAULT_PROJECT_CODE == projectCode;

        List<MenuDTO> resultList = new ArrayList<>();
        Map<String, Object> operationItemMap = new HashMap<>(16);
        //查询用户拥有的操作项
        List<TabSysMenuItem> menuItemList = menuItemService.listMenuItemByMenuId(authItemIdList);
        if (CollectionUtils.isNotEmpty(menuItemList)) {
            for (TabSysMenuItem menuItem : menuItemList) {
                operationItemMap.put(menuItem.getItemId().toString(), "1");
            }
        }
        //获取配置在权限项内的操作项数据
        List<MenuDTO> operationItemListOfAuthItem = getOperationItemOfAuthItem();
        //已配置的操作项需要权限控制
        //当时超级管理员的时候，operationItemMap是没有值的，因此会一直有效。
        if (CollectionUtils.isNotEmpty(operationItemListOfAuthItem)) {
            for (MenuDTO menuDTO : operationItemListOfAuthItem) {
                if (!operationItemMap.containsKey(menuDTO.getMenuId().toString())
                        || (isNeedValidVersionCode && StringUtils.isBlank(versionCode))
                        || (isNeedValidVersionCode && !menuDTO.getMenuVersion().contains("_" + versionCode + "_"))) {
                    //如果不存在，或者操作项的版本不对
                    //则是没有权限的操作项
                    resultList.add(menuDTO);
                }
            }
        }
        return resultList;
    }

    private ServiceResponse<List<MenuDTO>> getAppNoAuthOperationItem(List<Integer> menuIdList, String appId, Integer userId) {
        TabSysUser tabSysUser = userService.getUserById(userId);
        if (tabSysUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID：" + userId + "用户不存在");
        }
        //用户拥有的菜单权限
        List<TabSysMenu> menuList = filterOperationUserDepartAuth(menuService.selectByIds(menuIdList), tabSysUser);
        List<MenuDTO> resultList = getNoAuthOperationItemByMenuIdList(menuList, MenuProjectConstants.APP_PROJECT_CODE, "");
//        //仅超管可用
//        if (!isSuperAdmin) {
//            List<TabSysMenu> adminOnlyOperationItemList = menuService.listAdminOnlyOperationItem();
//            if (CollectionUtils.isNotEmpty(adminOnlyOperationItemList)) {
//                resultList.addAll(EntityUtil.changeEntityListNew(MenuDTO.class, adminOnlyOperationItemList));
//            }
//        }
        //过滤应用
        List<MenuDTO> tempResult = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(resultList)) {
            for (MenuDTO menuDTO : resultList) {
                if (menuDTO.getProject().equals(appId)) {
                    tempResult.add(menuDTO);
                }
            }
        }
        return ServiceResponse.success(tempResult);
    }
}
