package com.gic.auth.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.LoginUserTypeEnum;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.ResourceDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.dto.UserResourceDTO;
import com.gic.auth.service.*;
import com.gic.auth.web.vo.ChannelResourceVO;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.Md5Util;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.base.EnterpriseInfo;
import com.gic.enterprise.base.MenuInfo;
import com.gic.enterprise.base.UserInfo;
import com.gic.enterprise.base.UserResourceInfo;
import com.gic.enterprise.constant.EnterpriseStatusTypeEnum;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.context.UserContext;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
public class LoginController {
    private final static Logger logger = LogManager.getLogger(LoginController.class);

    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private OperationUserApiService operationUserApiService;
    @Autowired
    private UserResourceApiService userResourceApiService;
    @Autowired
    private ResourceApiService resourceApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;

    @RequestMapping("list-enterprise-by-phone")
    public RestResponse listUserEnterprise(String phoneNumber, String nationCode) {
        if (StringUtils.isBlank(phoneNumber)) {
            return EnterpriseRestResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        if (StringUtils.isBlank(nationCode)) {
            nationCode = Constants.NATION_CODE;
        }
        ServiceResponse<List<UserDTO>> listServiceResponse = this.userApiService.listUserByPhoneNumber(phoneNumber,
                nationCode);
        if (listServiceResponse.isSuccess()) {
            List<UserDTO> result = listServiceResponse.getResult();
            if (CollectionUtils.isEmpty(result)) {
                return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "用户不存在");
            } else {
                List<Integer> idList = new ArrayList<>();
                for (UserDTO userDTO : result) {
                    //gic管理员，不查询运营实施
                    if (userDTO.getLoginType().intValue() == LoginUserTypeEnum.GIC_USER.getCode()) {
                        idList.add(userDTO.getEnterpriseId());
                    }
                }
                ServiceResponse<List<EnterpriseDTO>> serviceResponse = this.enterpriseApiService
                        .listEnterpriseByIds(idList);
                if (serviceResponse.isSuccess()) {
                    List<EnterpriseDTO> list = serviceResponse.getResult();
                    if (CollectionUtils.isEmpty(list)) {
                        return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "用户不存在");
                    } else {
                        return RestResponse.success(list);
                    }
                }
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("login")
    public RestResponse login(String nationCode, String phoneNumber, Integer enterpriseId, String password) {
        if (StringUtils.isBlank(password) || StringUtils.isBlank(phoneNumber) || enterpriseId == null) {
            return EnterpriseRestResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        if (StringUtils.isBlank(nationCode)) {
            nationCode = Constants.NATION_CODE;
        }
        long startTime = System.currentTimeMillis();
        Md5Util md5 = new Md5Util();
        // password 自身作为盐值
        if (password.length() != 32) {
            password = md5.encrypt(password + password);
        }
        ServiceResponse<UserDTO> login = this.userApiService.login(nationCode, phoneNumber, enterpriseId, password);
        if (login.isSuccess()) {
            UserDTO userDTO = login.getResult();
            if (userDTO == null) {
                return EnterpriseRestResponse.failure(ErrorCode.LOGIN_ERR);
            }
            UserDetail userDetail = new UserDetail();
            userDetail.setUserId(userDTO.getUserId());
            userDetail.setUserInfo(EntityUtil.changeEntityNew(UserInfo.class, userDTO));
            ServiceResponse<EnterpriseDTO> enterprise = this.enterpriseApiService.getEnterpriseById(enterpriseId);
            if (enterprise.isSuccess()) {
                EnterpriseDTO enterpriseDTO = enterprise.getResult();
                Integer serviceStatus = enterpriseDTO.getServiceStatus();
                //如果停用
                if (serviceStatus != null && serviceStatus.intValue() == EnterpriseStatusTypeEnum.DISABLE.getCode()) {
                    return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "您的GIC商户后台已被停用，请及时联系达摩项目经理！");
                }
                if (enterpriseDTO != null) {
                    userDetail.setEnterpriseInfo(EntityUtil.changeEntityNew(EnterpriseInfo.class, enterpriseDTO));

                    ServiceResponse<List<MenuDTO>> menuResult = menuApiService.listUserMenu(userDetail.getUserId(),
                            enterpriseDTO.getVersionCode());
                    if (menuResult.isSuccess()) {
                        List<MenuDTO> menuList = menuResult.getResult();
                        List<MenuInfo> menuInfoList = EntityUtil.changeEntityListNew(MenuInfo.class, menuList);
                        Map<String, Object> map = new HashMap<>(16);
                        if (CollectionUtils.isNotEmpty(menuInfoList)) {
                            for (MenuInfo menuDTO : menuInfoList) {
                                if (StringUtils.isBlank(menuDTO.getMenuUrl())) {
                                    continue;
                                }
                                map.put(menuDTO.getMenuUrl(), menuDTO);
                            }
                        }
                        //查询全部的操作模块
                        Map<String, MenuInfo> moduleMap = new HashMap<>(16);
                        ServiceResponse<List<MenuDTO>> allGicMenu = menuApiService
                                .getAllMenuNotTree(enterpriseDTO.getVersionCode());
                        if (allGicMenu.isSuccess()) {
                            List<MenuDTO> temp = allGicMenu.getResult();
                            List<MenuInfo> tempMenuInfoList = EntityUtil.changeEntityListNew(MenuInfo.class, temp);
                            if (CollectionUtils.isNotEmpty(tempMenuInfoList)) {
                                for (MenuInfo menuDTO : tempMenuInfoList) {
                                    if (StringUtils.isBlank(menuDTO.getMenuUrl())) {
                                        continue;
                                    }
                                    moduleMap.put(menuDTO.getProjectUrlForWeb() + menuDTO.getMenuUrl(), menuDTO);
                                }
                            }
                            userDetail.setMenuInfoList(tempMenuInfoList);
                        }
                        //设置用户资源组
                        this.setUserResource(userDetail, userDetail.getUserId());
                        //塞值
                        userDetail.setMenuUrlMap(map);
                        userDetail.setModuleUrlMap(moduleMap);
                    }
                    //塞缓存
                    UserDetailUtils.setUserDetail(userDetail);
                    logger.info("登录所花时间:{}", System.currentTimeMillis() - startTime);
                    return RestResponse.success(userDetail);
                }
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("get-login-user-info")
    public RestResponse getLoginUserINfo() {
        return RestResponse.success(UserDetailUtils.getUserDetail().getUserInfo());
    }

    @RequestMapping("logout")
    public RestResponse logout() {
        UserDetailUtils.destory();
        return RestResponse.success();
    }

    /**
     * 运营平台单点登录
     * @Title: login 
     * @Description:
      * @author guojuxing
     * @param request
     * @param response 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @IgnoreLogin
    @RequestMapping("login-for-operation")
    public RestResponse login(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String redirectUrl = request.getParameter("redirectUrl");
        String token = request.getParameter("token");
        response.setStatus(302);
        if (UserContext.getContext().getByToken(token).getUserInfo() != null) {
            //已有登录信息
            response.setHeader("location", redirectUrl);
            //塞缓存
            UserDetailUtils.setUserDetail(UserContext.getContext().getByToken(token));
            return RestResponse.success();
        }
        return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "token错误");
    }

    private void setUserResource(UserDetail detail, Integer userId) {
        ServiceResponse<UserResourceDTO> resource = this.userResourceApiService.getResourceByUserId(userId);
        detail.setUserResourceInfo(new UserResourceInfo());
        if (resource.isSuccess() && resource.getResult() != null) {
            ServiceResponse<ResourceDTO> serviceResource = this.resourceApiService
                    .getResource(resource.getResult().getResourceId());
            if (serviceResource.isSuccess() && serviceResource.getResult() != null) {
                UserResourceInfo userResourceInfo = new UserResourceInfo();
                userResourceInfo.setUserResource(serviceResource.getResult().getUserResource());
                userResourceInfo.setStoreResource(serviceResource.getResult().getStoreResource());
                userResourceInfo.setGoodsResourceId(serviceResource.getResult().getGoodsResourceId());
                userResourceInfo.setAppResource(serviceResource.getResult().getAppResource());
                userResourceInfo.setOrderResource(serviceResource.getResult().getOrderResource());
                userResourceInfo.setSmsSignResource(serviceResource.getResult().getSmsSignResource());

               // 取的渠道资源
                String channelResourceJson = serviceResource.getResult().getChannelResourceJson();
                if (StringUtils.isNotBlank(channelResourceJson)) {
                    ChannelResourceVO channelResourceVO = JSON.parseObject(channelResourceJson, ChannelResourceVO.class);
                    userResourceInfo.setCardConfigIdList(channelResourceVO.getCardConfigIdList());
                    userResourceInfo.setAppletConfigIdList(channelResourceVO.getAppletConfigIdList());
                    userResourceInfo.setServiceConfigIdList(channelResourceVO.getServiceConfigIdList());
                }

                detail.setUserResourceInfo(userResourceInfo);
            }
        }
    }
}
